import java.io.InputStream;
import java.net.URLEncoder;
import java.nio.charset.Charset;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

import javax.crypto.Mac;
import javax.crypto.spec.SecretKeySpec;

import org.apache.http.HttpEntity;
import org.apache.http.NameValuePair;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.message.BasicNameValuePair;
import org.apache.http.util.EntityUtils;

public class SignatureDemo {
	public static void main(String[] args) throws Exception{
		String appKey = "appKey_xxx";//请填写申请的appkey
		String appSecret = "appSecret_xxx";//请填写您申请的示例
		String apiName = "gpsp.entity.createEntity";// 访问的API接口名称
		String apiPath = "/rest/" + apiName + "/" + appKey + "/0";// API路径，注意没有问号
		// 参数列表(加签的时候需要进行字典序升序排序)
		Map<String, String> paramMap = new HashMap<String, String>();
		String entity="{\"entityName\":\"沪ABCD\",\"entityType\":\"小汽车\",\"attributes\":{\"color\":\"黑色\",\"driver\":\"千小寻\"}}";
		paramMap.put("entity", entity);
		// 毫秒级时间戳，一定是毫秒级！重要！！！
		String timestamp = String.valueOf(System.currentTimeMillis());
		String signatureStr = SignatureDemo.doHmacSHA2(apiPath, paramMap, appSecret, timestamp);
		System.out.println("加签值："+signatureStr);//打印sign值
		
		String qxwzUrl = "http://api.qxwz.com";//千寻的服务器地址
		String queryUrl = qxwzUrl + apiPath + "?_sign=" + signatureStr;
		
        //发送http请求，获取返回值
        doHttpPost(queryUrl,paramMap,timestamp);
	}
		
	public static void doHttpPost(String url,Map<String, String> paramMap, String timestamp) throws Exception {
		CloseableHttpClient httpClient = HttpClients.createDefault();	
		HttpPost httpPost = new HttpPost(url);
        httpPost.setHeader("Accept-Encoding", "gzip, deflate");
        httpPost.setHeader("wz-acs-timestamp", timestamp);//此处注意加上时间戳，否则http请求将无效
      	List<NameValuePair> nvps = new ArrayList<NameValuePair>();  
        for (Map.Entry<String, String> entry : paramMap.entrySet()){
             if (entry.getValue() == null) continue;
              	nvps.add(new BasicNameValuePair(entry.getKey(), entry.getValue()));
        }
        httpPost.setEntity(new UrlEncodedFormEntity(nvps, "UTF-8"));//处理中文编码的问题

        
		try {
			CloseableHttpResponse response = (CloseableHttpResponse) httpClient.execute(httpPost); 
			try {  
                HttpEntity entity = response.getEntity();   
                if (entity != null) {    
                    System.out.println("返回值："+EntityUtils.toString(entity));  
                    /*
                     * 此处针对返回结果进行不同的处理
                     */
                    EntityUtils.consume(entity);  
                }  
            } finally {  
                response.close();  
            } 

		} catch (Exception e) {
			e.printStackTrace();
		}finally {
			if (httpPost != null) {
				httpPost.releaseConnection();//释放资源
			}
			if(httpClient != null){
				httpClient.close();//释放资源
			}
		}
	}

	/*
	 * 将字节数组转换成16进制字符串
	 * */
	public static String encodeHexStr(final byte[] bytes){
		if (bytes == null) {
			return null;
		}
		char[] digital  = "0123456789ABCDEF".toCharArray();
		char[] result = new char[bytes.length * 2];
		for (int i = 0; i < bytes.length; i++) {
			result[i * 2] = digital[(bytes[i] & 0xf0) >> 4];
			result[i * 2 + 1] = digital[bytes[i] & 0x0f];
		}
		return new String(result);
	}

	/*
	 * 加签算法
	 * */
	public static <T> String doHmacSHA2(String path, Map<String, T> params, String key, String timestamp) {
		
		List<Map.Entry<String, T>> parameters = new ArrayList<Map.Entry<String,T>>(params.entrySet());
		SecretKeySpec signingKey = new SecretKeySpec(key.getBytes(), "HmacSHA256");
		Charset CHARSET_UTF8 = Charset.forName("UTF-8");
		Mac mac;
		try {
			mac = Mac.getInstance("HmacSHA256");
			mac.init(signingKey);
		} catch (NoSuchAlgorithmException e) {
			throw new IllegalStateException(e.getMessage(), e);
		} catch (InvalidKeyException e) {
			throw new IllegalStateException(e.getMessage(), e);
		}
		if(path != null && path.length() > 0){
			mac.update(path.getBytes(CHARSET_UTF8));
		}
		if(parameters != null){
			Collections.sort(parameters, new MapEntryComparator<String, T>());
			for (Map.Entry<String, T> parameter : parameters) {
				byte[] name = parameter.getKey().getBytes(CHARSET_UTF8);
				Object value = parameter.getValue();
				if(value instanceof Collection){
					for (Object o : (Collection)value){
						mac.update(name);
						if(o != null){
							mac.update(o.toString().getBytes(CHARSET_UTF8));
						}
					}
				}else{
					mac.update(name);
					if(value != null){
						mac.update(value.toString().getBytes(CHARSET_UTF8));
					}
				}
			}
		}
		if(timestamp != null && timestamp.length() > 0){
			mac.update(timestamp.toString().getBytes(CHARSET_UTF8));
		}
		return encodeHexStr(mac.doFinal());
	}
}

/*
 * Map参数排序类
 * */

class MapEntryComparator<K, V> implements Comparator<Map.Entry<K, V>> {

    @Override
    public int compare(Entry<K, V> o1, Entry<K, V> o2) {
        if (o1 == o2) {
            return 0;
        }
        final String k1 = o1.getKey().toString();
        final String k2 = o2.getKey().toString();
        int l1 = k1.length();
        int l2a = k2.length();
        for (int i = 0; i < l1; i++) {
            char c1 = k1.charAt(i);
            char c2;
            if (i < l2a) {
                c2 = k2.charAt(i);
            } else {
                return 1;
            }
            if (c1 > c2) {
                return 1;
            } else if (c1 < c2) {
                return -1;
            }
        }
        if (l1 < l2a) {
          return -1;
        }else if(l1 == l2a) {
          return 0;
        }else {
          return -1;
        }
    }
}